﻿/*
Dojo widget for rendering a QBank Media Selector.
*/

var currentQBankPickerWidget = null;


define([
    "dojo/_base/array",
    "dojo/_base/connect",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dijit/_CssStateMixin",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "dijit/form/Textarea",

    "epi/epi",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/dependency",
    "epi/routes",
    "epi-cms/_ContentContextMixin"
],
function (
    array,
    connect,
    declare,
    lang,

    _CssStateMixin,
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,

    Textarea,
    epi,
    _ValueRequiredMixin,
    dependency,
    routes,
    _ContentContextMixin
) {
    // These variables are needed to keep a reference to a rest store and to keep a reference to the widget by itself
    // var currentSelectWidget = null;
    var activeWidgets = {};
    var theStore = null;
    var mediaStore = null;
    var settingsStore = null;
    var contentId = null;

    return declare("qbank.editors.MediaSelector", [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _CssStateMixin, _ValueRequiredMixin, _ContentContextMixin], {

        templateString: "<div class=\"epi-formsWidgetWrapper\">\
                            <div class=\"dijitReset dijitInputField dijitInputContainer dijitInline epi-resourceInputContainer epi-noValue\" style=\"text-align:center;\">\
                                <!--<h3>QBank Media Picker</h3>-->\
                                <div data-dojo-attach-point=\"stateNode, tooltipNode\">\
                                    <div data-dojo-attach-point=\"viewArea\" style=\"backgroundcolor:#fff0f0; margin:5px 0px;\"></div>\
                                    <div data-dojo-attach-point=\"selectArea\" style=\"backgroundcolor:#fff0f0; margin:5px 0px; display:none;\"></div>\
                                    <div data-dojo-attach-point=\"loaderArea\" style=\"display:none; position:absolute; width:20%; border:solid 1px #a0a0a0; left:35%; top: 20%; background-color: #ffffff; padding:0% 5%; border-radius: 6px; \"></div>\
                                    <div style=\"display:block;\">\
                                        <div data-dojo-attach-point=\"textArea\" data-dojo-type=\"dijit.form.Textarea\" style=\"margin-top:10px; width: 100%; min-height:20px; display:none;\"></div>\
                                    </div>\
                                </div>\
                            </div>\
                        </div>",

        baseClass: "epiPicker",

        intermediateChanges: false,

        value: null,

        multiple: true,

        modulePath: '',

        onChange: function (value) {
            // Event
        },

        constructor: function () {
            //currentSelectWidget = this;
        },

        postCreate: function () {
            // call base implementation
            this.inherited(arguments);

            var storePath = this._getRestPath("");

            this.modulePath = storePath.substr(0, storePath.search(/qbankmodule/i)) + 'qbankmodule/';

            // Save a reference
            activeWidgets[this._widgetId()] = this;
            currentQBankPickerWidget = null;

            // Init textarea and bind event
            this.textArea.set("intermediateChanges", this.intermediateChanges);
            this.connect(this.textArea, "onChange", this._onTextAreaChanged);

            // To simplify things, we save a reference to this widget object
            // to be called inside asyncronus calls for example
            //currentSelectWidget = this;

            if (theStore == null) {
                //Register the store
                var registry = dependency.resolve("epi.storeregistry");;
                registry.create("qbankPicker.qbankconverter", this._getRestPath("qbankconverter"));
                theStore = registry.get("qbankPicker.qbankconverter");
            }
            if (mediaStore == null) {
                //Register the media store
                var registry = dependency.resolve("epi.storeregistry");;
                registry.create("qbankPicker.qbankmediastore", this._getRestPath("qbankmediastore"));
                mediaStore = registry.get("qbankPicker.qbankmediastore");
            }

            if (settingsStore == null) {
                //Register the settings store
                var registry = dependency.resolve("epi.storeregistry");;
                registry.create("qbankPicker.qbanksettingsstore", this._getRestPath("qbanksettingsstore"));
                settingsStore = registry.get("qbankPicker.qbanksettingsstore");
            }

            var propertyName = typeof (this.name) == 'undefined' ? this.parent.propertyName : this.name;
            this.contentId = this._currentContext.id;
            var parameters = { 'widgetId': this._widgetId(), 'currentContentId': this.contentId, 'propertyName': propertyName, 'currentMediaId': this.value };
            dojo.when(settingsStore.query(parameters), function (settingsDataJson) {
                var settingsData = JSON.parse(settingsDataJson);
                activeWidgets[settingsData.widgetId]._settingsData = settingsData;
                activeWidgets[settingsData.widgetId]._renderImageView();
            });

        },

        _settingsData: null,

        _getRestPath: function (name) {
            return routes.getRestPath({ moduleArea: "qbankmodule", storeName: name });
        },


        isValid: function () {
            // summary:
            //    Check if widget's value is valid.
            // tags:
            //    protected, override
            return !this.required || this.value != null;
        },

        // Setter for value property
        _setValueAttr: function (value) {
            id = '';
            if (value == null) {
                id = 0;
            }
            else if (!isNaN(value.id)) {
                id = value.id;
            }
            else {
                id = value;
            }

            this._setValue(id, true);
        },

        _setValue: function (value, updateTextarea) {
            var medRef = { 'id': parseInt(value), 'originalMediaId': 233 };

            if (this._started && epi.areEqual(this.value, value)) {
                return;
            }

            // set value to this widget (and notify observers)
            this._set("value", medRef.id + '');

            // set value to textarea
            updateTextarea && this.textArea.set("value", value + '');

            if (this._started && this.validate()) {
                // Trigger change event
                this.onChange('' + medRef.id + '');

                // Force focus change to allow EPiServer to trigger the model update and auto save
                if (this.focus) {
                    this.onBlur();
                    this.focus(this.domNode);
                }
            }


        },

        _setReadOnlyAttr: function (value) {
            this._set("readOnly", value);
            this.textArea.set("readOnly", value);
        },

        // Setter for intermediateChanges
        _setIntermediateChangesAttr: function (value) {
            this.textArea.set("intermediateChanges", value);
            this._set("intermediateChanges", value);
        },

        // Event handler for textarea
        _onTextAreaChanged: function (value) {
            //            this._setValue( { 'dataToShow': value }, false);
        },

        _updateImage: function () {
            var id = parseInt(this.textArea.value);
            if (isNaN(id) || id == 0) {
                $("#btnAddImage_" + this._widgetId()).attr('src', this.modulePath + 'ClientResources/Images/qbankadd.png');
            }
            else {
                var parameters = { 'id': id, 'widgetId': this._widgetId() };
                dojo.when(mediaStore.query(parameters), function (mediaData) {
                    if (mediaData.url != '') {
                        $("#btnAddImage_" + mediaData.widgetId).attr('src', mediaData.url);
                    }
                    else {
                        $("#btnAddImage_" + mediaData.widgetId).attr('src', activeWidgets[mediaData.widgetId].modulePath + 'ClientResources/Images/qbankadd.png');
                    };
                });
            }
        },

        //
        // To be able to handle multiple concurrent widgets on a page we need to
        // be able to identify what widget is the curren right now
        //
        _widgetId: function () {
            if (this.id == null || this.id == '') {
                return "999";
            }
            var segments = this.id.split('_');
            return segments[segments.length - 1];
        },

        _selectedMediaId: 0,

        _renderLoaderSymbol: function () {
            $(this.viewArea)
        },

        //
        // This method will render the media tool area with Add and Edit in EPiServer
        //
        _renderImageView: function () {

            var translate = function (languageDictionary, key, fallback) {
                var baseText = [{ "translation": fallback }];
                if (languageDictionary != undefined) {
                    baseText = $.grep(languageDictionary, function (e) { return e.Name == key; });
                }
                return baseText != undefined && baseText.length == 1 ? baseText[0].Translation : fallback;
            };

            var html = '';
            html += '<img id="btnAddImage_' + this._widgetId() + '" src="' + this.modulePath + 'ClientResources/Images/qbankadd.png" title="' + translate(this._settingsData.languageDictionary, "qbankeditortooltip", "Click to add or modify media") + '" style="margin-left:5px; max-width:100%; max-height:200px;">';
            html += '<div style="text-alaign:center;">';
            html += '<input id="btnDelete_' + this._widgetId() + '" type="button" value="' + translate(this._settingsData.languageDictionary, "qbankeditordelete", "Delete") + '" style="width:120px; padding:5px; margin:10px;" />'
            html += '<input id="btnEdit_' + this._widgetId() + '" type="button" value="' + translate(this._settingsData.languageDictionary, "qbankeditoredit", "Edit") + '" style="width:120px; padding:5px; margin:10px;" />'
            html += '</div>';

            var loaderHtml = '<img id="btnLoaderImage_' + this._widgetId() + '" src="' + this.modulePath + 'ClientResources/Images/qbank-loader.gif" title="" style="width:100%;" >';

            this.viewArea.innerHTML = html;
            this.selectArea.innerHTML = '';
            this.loaderArea.innerHTML = loaderHtml;

            $(this.viewArea).show();
            $(this.selectArea).hide();
            $(this.loaderArea).hide();

            this._updateImage();

            $("#btnAddImage_" + this._widgetId()).click(function () {
                if (currentQBankPickerWidget == null) {
                    currentQBankPickerWidget = activeWidgets[this.id.split('_')[1]];
                    currentQBankPickerWidget._renderWrapperDialog();
                }
            })

            $("#btnEdit_" + this._widgetId()).click(function () {
                if (currentQBankPickerWidget == null) {
                    currentQBankPickerWidget = activeWidgets[this.id.split('_')[1]];
                    currentQBankPickerWidget._renderWrapperDialog();
                }
            })

            $("#btnDelete_" + this._widgetId()).click(function () {
                if (currentQBankPickerWidget == null) {
                    currentQBankPickerWidget = activeWidgets[this.id.split('_')[1]];
                    currentQBankPickerWidget._selectedMediaId = 0;
                    currentQBankPickerWidget._setValue('0', true);
                    currentQBankPickerWidget._updateImage();
                    currentQBankPickerWidget = null;
                }
            })
        },

        _currentWidth: 0,
        _currentHeight: 0,
        _autoresize: function (areaTag) {
            if (currentQBankPickerWidget != null) {
                setTimeout(function () {
                    if (currentQBankPickerWidget != null) {

                        var dialogSize = currentQBankPickerWidget.__proto__._getSize(areaTag);

                        if (currentQBankPickerWidget._currentWidth != dialogSize.width || currentQBankPickerWidget._currentHeight != dialogSize.height) {
                            console.info("Resizing...");
                            var qbankPickerWrapper = $("#qbankPickerWrapper");
                            var qbankPickerIframe = qbankPickerWrapper.find("iframe");

                            qbankPickerWrapper.width(dialogSize.width);
                            qbankPickerWrapper.height(dialogSize.height);
                            qbankPickerIframe.width(dialogSize.width);
                            qbankPickerIframe.height(dialogSize.height);

                            currentQBankPickerWidget._currentWidth = dialogSize.width;
                            currentQBankPickerWidget._currentHeight = dialogSize.height;
                        }

                        currentQBankPickerWidget.__proto__._autoresize(areaTag);
                    }
                }, 300);
            }
        },

        _getSize: function (areaTag) {
            dialogWidth = 840;
            dialogHeight = 600;

            // Before CMS 8
            //var contentPane = $(areaTag).closest(".epi-editorViewport");
            //var formsPane = (areaTag).closest(".epi-formsWidgetWrapper");

            //if (contentPane.length > 0) {
            //    dialogWidth = contentPane.width() - 41;
            //    dialogHeight = contentPane.height() - 150;
            //}
            //else if (formsPane.length > 0)
            //{
            //    dialogWidth = formsPane.width() - 200;
            //    dialogHeight = 500;
            //}

            var $mainLayout = $('div[data-dojo-attach-point="mainLayoutContainer"]');
            if ($mainLayout.length > 0) {
                dialogWidth = $mainLayout.width() - 200;
            }

            return { 'width': dialogWidth, 'height': dialogHeight };
        },

        //
        // This method will render the Wrapper for the QBank Picker and start waiting
        // for the Picker to open
        //
        _renderWrapperDialog: function () {

            if (!currentQBankPickerWidget._settingsData.pickerSettings.host) {
                alert("The Service Host is missing. Setup QBank correctly in Admin mode.");
            }
            else {

                this._selectedMediaId = 0;

                var areaTag = $("#" + this.id);
                var dialogSize = this._getSize(areaTag);

                var html = "";
                html += '<div id="qbankPickerWrapper" style="/*width:' + dialogSize.width + 'px; min-height:400px; padding: 10px;border:solid 1px #999999; border-radius: 4px; */">';
                html += '  <iframe id="qbankPickerIframe_' + this._widgetId() + '" src="' + this.modulePath + 'ClientResources/QBankConnect.html" style="display:block; width:' + dialogSize.width + 'px; height:' + dialogSize.height + 'px;" >No iframe support</iframe>';

                html += '</div>';

                this.viewArea.innerHTML = '';
                this.selectArea.innerHTML = html;

                $(this.viewArea).hide();
                $(this.selectArea).show();

                this._autoresize(areaTag);
                this.domNode.scrollIntoView();

                $("#qbankPickerIframe_" + this._widgetId()).load(function () {
                    var widgetId = this.id.split('_')[1];
                    var currentWidget = activeWidgets[widgetId];

                    setTimeout(function () { currentWidget._pickerInitDialogDisplay() }, 100);
                });
            }
        },

        _pickerDisplayTimeout: null,

        _pickerInitDialogDisplay: function () {
            var timeout = 5;
            var d1 = new Date(), d2 = new Date(d1);
            d2.setSeconds(d1.getSeconds() + timeout);
            this._pickerDisplayTimeout = d2;

            var currentWidget = this;

            setTimeout(function () { currentWidget._pickerDisplayWhenReady() }, 0);
        },

        _currentMediaPicker: null,

        //
        // This method will acctually call the Media Picker and display it based
        // on the settings provided
        //
        _pickerDisplayWhenReady: function () {

            // These are not used yet
            var formats = {};
            var currentMedia = {};
            var uiSettings = {};
            var mediaFilter = {};

            if (currentQBankPickerWidget._settingsData != null) {
                if (currentQBankPickerWidget._settingsData.pickerSettings != null) {
                    if (currentQBankPickerWidget._settingsData.pickerSettings.mediaFormats != null) {
                        formats = currentQBankPickerWidget._settingsData.pickerSettings.mediaFormats;
                    }
                    if (currentQBankPickerWidget._settingsData.pickerSettings.mediaFilter != null) {
                        mediaFilter = currentQBankPickerWidget._settingsData.pickerSettings.mediaFilter;
                    }
                    if (currentQBankPickerWidget._settingsData.pickerSettings.uiSettings != null) {
                        uiSettings = currentQBankPickerWidget._settingsData.pickerSettings.uiSettings;
                    }
                }
                if (currentQBankPickerWidget._settingsData.currentMedia != null) {
                    currentMedia = currentQBankPickerWidget._settingsData.currentMedia;
                }

                var pickerWindow = document.getElementById('qbankPickerIframe_' + currentQBankPickerWidget._widgetId()).contentWindow;
                var qbcConfig = {
                    api: {
                        host: currentQBankPickerWidget._settingsData.pickerSettings.host,
                        access_token: currentQBankPickerWidget._settingsData.AccessToken
                    },


                    deploymentSite: mediaFilter.DeploymentSiteId,
                    rootFolderId: mediaFilter.RootFolderId
                };

                var qbankConnector = new pickerWindow.QBankConnector(qbcConfig);

                var cropping = !uiSettings.HideCropping;


                var cropping = currentMedia.Crop;

                if (uiSettings.HideCropping) {
                    cropping = false;
                }
                else if (cropping) {
                    if (cropping.x == 0 && cropping.y == 0 && cropping.w == 0 && cropping.h == 0) {
                        cropping = null;
                    }
                }

                var originalMedia = currentMedia.OriginalMedia;
                if ((undefined == originalMedia || originalMedia == 0) && undefined != currentQBankPickerWidget._currentOriginalMediaId && currentQBankPickerWidget._currentOriginalMediaId != 0) {
                    originalMedia = currentQBankPickerWidget._currentOriginalMediaId;
                }

                currentQBankPickerWidget._currentMediaPicker = new qbankConnector.mediaPicker({
                    header: uiSettings.ShowHeader,
                    container: $(pickerWindow.document.getElementById("qbankwrapper")),
                    requestedTemplates: formats.MediaTemplates,
                    mediaId: originalMedia,
                    modules: {
                        imageTool: {
                            crop: cropping
                        }
                    },
                    pageUrl: currentQBankPickerWidget._settingsData.pickerSettings.pageUrl,


                    onSelect: function (media, basePickerTemplates) {

                        // A Media Item has been selected in QBank

                        var pickerTemplates = [];
                        for (var index in basePickerTemplates) {
                            pickerTemplates.push({ "baseTemplate": basePickerTemplates[index].template, "url": basePickerTemplates[index].url })
                        };

                        currentQBankPickerWidget._selectedMediaId = media.mediaId;
                        currentQBankPickerWidget._startConversion(media, pickerTemplates);
                        setTimeout(function () { currentQBankPickerWidget._waitForConversion(currentQBankPickerWidget._widgetId()) }, 10);
                    },
                    onClose: function () {
                        currentQBankPickerWidget._closeMediaPicker(false);
                    }
                });

                pickerWindow.QbankShowPicker();

            }
            else if (this._pickerDisplayTimeout > new Date()) {
                // Wait until the settings arrive
                setTimeout(function () { currentQBankPickerWidget._pickerDisplayWhenReady() }, 200);
            }

        },

        _closeMediaPicker: function (forceClose) {
            if (this._currentMediaPicker) {
                if (forceClose) {
                    this._currentMediaPicker.close();
                }
                this._renderImageView();
                currentQBankPickerWidget = null;
            }
        },

        _convertedId: 0,
        _conversionTimeout: null,

        //
        // This method will do the acctual creation and conversion
        // of the EPiServer Media by calling the server
        // 
        _startConversion: function (media, pickerTemplates) {

            var timeout = 60;
            this._convertedId = 0;
            var d1 = new Date(), d2 = new Date(d1);
            d2.setSeconds(d1.getSeconds() + timeout);
            this._conversionTimeout = d2;

            var widgetId = this._widgetId();

            activeWidgets[widgetId]._currentOriginalMediaId = media.mediaId;

            // Read the data
            var pickerTemplatesJson = JSON.stringify(pickerTemplates);
            var parameters = { 'currentContentId': this.contentId, 'widgetId': widgetId, 'episerverMediaId': 0, 'qbankMediaId': media.mediaId, 'pickerTemplatesJson': pickerTemplatesJson, 'useTinyTemplate': false };
            dojo.when(theStore.query(parameters), function (convertedMedia) {

                var id = convertedMedia.id;
                if (id != undefined && id != null) {
                    activeWidgets[convertedMedia.widgetId]._convertedId = convertedMedia.id;
                }
                else {
                    alert(convertedMedia); // Error message
                }

                activeWidgets[convertedMedia.widgetId]._convertedId = convertedMedia.id;
            });
        },

        //
        // This method showing a progressbar is initally called when the user selects a media item
        // and the media is being converted and saved by EPiServer.
        // When EPiServer is done, the progressbar is hidden.
        //
        _waitForConversion: function (widgetId) {
            var currentWidget = activeWidgets[widgetId];
            if (currentWidget._convertedId != 0 || currentWidget._conversionTimeout < new Date()) {
                if (currentWidget._conversionTimeout < new Date()) {
                    alert("Timeout, no media was converted.")
                }
                currentWidget._setValue(currentWidget._convertedId + '', true);
                currentWidget._renderImageView();
                currentWidget._currentMediaPicker = null;
                currentQBankPickerWidget = null;
                $(this.loaderArea).hide();
            }
            else {
                $('#dataArea').html('<h3>Converting... <br /> (' + new Date() + ')</h3>');
                $(this.loaderArea).show();
                setTimeout(function () { currentWidget._waitForConversion(widgetId) }, 500);
            }
        }

    });
});
