﻿; (function (window, document, QBank, jQueryQ, undefined) {

    // QBank support class for TinyMCE in EPiServer
    QBank.tiny = (function () {
        var qBankWrapperTag = 3000;
        var supportObject;
        var dialogCounter = 0;
        var editor;
        var settingsData;
        var currentMediaUrl;
        var epiPageId = 0;
        var appRootPath = 0;

        var jsonEPiServerStoreFilter = function (data) {
            var version = jQueryQ.fn.jquery;
            if (version && version.length > 0 && version[0] != '1') {
                // Later jQuery version can´t handle json response from EPiServer Rest Store
                // if jQuery 2 or later is used. This should not happen as long as the epiJQuery
                // legacy class is using jQuery 1.7.2

                if (data.indexOf('{}&&') == 0) {
                    var startPos = data[4] == '"' ? 5 : 4;
                    var stringLength = data[4] == '"' ? data.length - 6 : data.length - 4;
                    var jsonData = data.substr(startPos, stringLength); // Remove the hijacking wrapper

                    // Clean up the content
                    jsonData = jsonData.replace(/\\\"/g, '_§1_'); // \"
                    jsonData = jsonData.replace(/\\\\/g, '_§2_'); // \\
                    jsonData = jsonData.replace(/\\r/g, '\r');
                    jsonData = jsonData.replace(/\\n/g, '\n');
                    jsonData = jsonData.replace(/_§1_/g, '"');
                    jsonData = jsonData.replace(/_§2_/g, '\\');

                    var cleanJsonObject = JSON.parse(jsonData);
                    data = JSON.stringify(cleanJsonObject);
                }
            }
            return data;
        }

        var support = function () {
            var self = this;

            // The EPiServer UI has some delay before the editor iframe is loaded
            // so the acctual loading will need to wait until it is realy created
            self.waitForDialog = function (callLimit) {
                if (callLimit != undefined && callLimit != null) {
                    dialogCounter = callLimit
                };

                var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                if (iframeWindow.length > 0 && iframeWindow[0].contentWindow.QBankConnector != null) {
                    self.loadSettingsAndRender();
                }
                else {
                    if (dialogCounter > 0) {
                        window.setTimeout(function () {
                            self.waitForDialog();
                        }, 100);
                        dialogCounter--;
                    }
                }
            },

            // The EPiServer iframe is open and now settings can be loaded and then
            // the QBank Picker dialog can be opened
            self.loadSettingsAndRender = function () {

                var widgetId = 0;
                var contentId = parseInt(epiPageId);
                var propertyName = '#tinymce';
                var currentMediaId = self.getMediaId(currentMediaUrl);

                var parameters = { 'widgetId': widgetId, 'currentContentId': contentId, 'propertyName': propertyName, 'currentMediaId': currentMediaId, 'currentMediaUrl': currentMediaUrl };
                jQueryQ.ajax({
                    url: appRootPath + "qbankmodule/Stores/qbanksettingsstore/",
                    type: "GET",
                    data: parameters,
                    contentType: "application/json",
                    dataType: "json",
                    dataFilter: jsonEPiServerStoreFilter,
                    error: function (data) {
                        alert("Problems loading the settings!")
                    },
                    success: function (settingsDataJson) {
                        // The settings are loaded - let´s render the dialog

                        if (typeof (settingsDataJson) == 'object') {
                            settingsData = settingsDataJson;
                        }
                        else {
                            settingsData = JSON.parse(settingsDataJson);
                        }
                        currentMediaId = settingsData.currentMedia.ContentId;

                        var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                        var dialogWindow = iframeWindow[0].contentWindow;

                        var loaderTag = "<div id=\"loaderArea\" style=\"display:none; position:absolute; width:10%; border:solid 1px #a0a0a0; left:44%; top: 20%; background-color: #ffffff; padding:0% 3%; border-radius: 6px; \">"
                        loaderTag += '<img id="btnLoaderImage" src="' + appRootPath + 'qbankmodule/ClientResources/Images/qbank-loader.gif" title="" style="width:100%;" >';
                        loaderTag += "</div>";
                        jQueryQ(loaderTag).insertBefore(iframeWindow);


                        var qbcConfig = {
                            api: {
                                host: settingsData.pickerSettings.host,
                                access_token: settingsData.AccessToken
                            },
                            deploymentSite: settingsData.pickerSettings.mediaFilter.DeploymentSiteId,
                            rootFolderId: settingsData.pickerSettings.mediaFilter.RootFolderId
                        };
                        var qbankConnector = new dialogWindow.QBankConnector(qbcConfig);

                        //
                        // Prepare the Property View in the Connector
                        //
                        self.prepareProperties();


                        // Turn default cropping off for now because scaling is not working correctly
                        //cropping = null;


                        var cropping = settingsData.currentMedia.Crop;

                        if (settingsData.pickerSettings.uiSettings.HideCropping) {
                            cropping = false;
                        }
                        else if (cropping) {
                            if (cropping.x == 0 && cropping.y == 0 && cropping.w == 0 && cropping.h == 0) {
                                cropping = null;
                            }
                        }

                        //
                        // Prepare the QBank Media Picker View in the Connector
                        //
                        var mediaPicker = new qbankConnector.mediaPicker({
                            header: settingsData.pickerSettings.uiSettings.ShowHeader,
                            requestedTemplates: settingsData.pickerSettings.mediaFormats.MediaTemplates,
                            container: jQueryQ(dialogWindow.document.getElementById("qbankwrapper")),
                            mediaId: settingsData.currentMedia.OriginalMedia,
                            modules: {
                                imageTool: {
                                    crop: cropping
                                }
                            },
                            pageUrl: settingsData.pickerSettings.pageUrl,

                            onSelect: function (media, basePickerTemplates) {
                                // Some media is selected and ready for the next step - Convert and Show Properties
                                var pickerTemplates = [];
                                for (var index in basePickerTemplates) {
                                    pickerTemplates.push({ "baseTemplate": basePickerTemplates[index].template, "url": basePickerTemplates[index].url })
                                };

                                self.convertAndShowProperties(media, pickerTemplates);
                            }
                        });

                        // New media - show the picker view
                        // Reopen media - show the properties view
                        if (currentMediaId != '') {
                            var mediaId = settingsData.currentMedia.OriginalMedia;
                            var propertyValueList = [];
                            var wrapperTag = jQueryQ(qBankWrapperTag);

                            propertyValueList.push({ 'key': 'url', 'value': currentMediaUrl });

                            // In this section all properties are fetched from the HTML 

                            var mediaTag = wrapperTag.find(".qbankmedia").andSelf();
                            if (mediaTag.length > 0) {
                                propertyValueList.push({ 'key': 'alttext', 'value': mediaTag.attr('alt') });
                                propertyValueList.push({ 'key': 'title', 'value': mediaTag.attr('title') });
                            }

                            var prop1 = wrapperTag.find(".qbankproperty1");
                            var prop2 = wrapperTag.find(".qbankproperty2");
                            var prop3 = wrapperTag.find(".qbankproperty3");

                            if (prop1.length > 0) {
                                propertyValueList.push({ 'key': 'qbankproperty1', 'value': prop1.text() });
                            }
                            if (prop2.length > 0) {
                                propertyValueList.push({ 'key': 'qbankproperty2', 'value': prop2.text() });
                            }
                            if (prop3.length > 0) {
                                propertyValueList.push({ 'key': 'qbankproperty3', 'value': prop3.text() });
                            }

                            var mediaType = "image";
                            var metaData = wrapperTag.find("[class*='qbankmetadata_']").andSelf();
                            if (metaData.length > 0) {
                                var parts = metaData[0].className.split(' ');
                                for (var index in parts) {
                                    if (parts[index].indexOf('qbankmetadata_') > -1) {
                                        var valueParts = parts[index].split('_');
                                        if (valueParts.length > 1) {
                                            mediaType = valueParts[1];
                                        }
                                        if (valueParts.length > 2) {
                                            propertyValueList.push({ 'key': 'templateid', 'value': valueParts[2] });
                                        }
                                        if (valueParts.length > 3) {
                                            propertyValueList.push({ 'key': 'templatename', 'value': valueParts[3] });
                                        }
                                        if (valueParts.length > 4) {
                                            propertyValueList.push({ 'key': 'viewstyle', 'value': valueParts[4] });
                                        }
                                    }
                                }
                            }

                            self.showProperties(propertyValueList, mediaType);
                        }
                        else {
                            self.showPicker();
                        }

                    },
                    failure: function () {
                        alert("Settings loader failure...");
                    }
                });

            };

            // Create the property area in the Connector
            self.prepareProperties = function () {
                var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                var dialogWindow = iframeWindow[0].contentWindow;

                dialogWindow.QbankPropertyLabelDisplay(settingsData,
                                function (propertyValueList, mediaType) {
                                    // OnUpdate

                                    console.info("## QBank - OnUpdate in 'dialogWindow.QbankPropertyLabelDisplay'");

                                    var activeEditor = window.tinymce.activeEditor;

                                    // Add support for Video and Documents
                                    var html = settingsData.viewTemplates.viewTemplateImage;
                                    if (mediaType == 'video') {
                                        html = settingsData.viewTemplates.viewTemplateVideo;
                                    }
                                    else if (mediaType == 'document') {
                                        html = settingsData.viewTemplates.viewTemplateDocument;
                                    }

                                    var propertyValue1 = self.getPropertyValue(propertyValueList, "qbankproperty1");
                                    var propertyValue2 = self.getPropertyValue(propertyValueList, "qbankproperty2");
                                    var propertyValue3 = self.getPropertyValue(propertyValueList, "qbankproperty3");
                                    var altText = self.getPropertyValue(propertyValueList, "alttext");
                                    var title = self.getPropertyValue(propertyValueList, "title");
                                    var url = self.getPropertyValue(propertyValueList, "url");
                                    var editUrl = self.getPropertyValue(propertyValueList, "editurl");
                                    var externalSource = self.getPropertyValue(propertyValueList, "externalsource"); // externalsource
                                    var templateId = self.getPropertyValue(propertyValueList, "templateid");
                                    var templateName = self.getPropertyValue(propertyValueList, "templatename");
                                    var mediaClassText = self.getPropertyValue(propertyValueList, "viewstyle");
                                    var mediaName = self.getPropertyValue(propertyValueList, "medianame");
                                    var mediaType = self.getPropertyValue(propertyValueList, "mediatype");
                                    var siteUrl = self.getPropertyValue(propertyValueList, "siteurl");

                                    html = html.replace(/{qbankproperty1}/g, propertyValue1).replace(/{qbankproperty2}/g, propertyValue2).replace(/{qbankproperty3}/g, propertyValue3);
                                    html = html.replace(/{templatename}/g, templateName).replace(/{templateid}/g, templateId);
                                    if (editUrl != null && editUrl != '') {
                                        html = html.replace(/{url}/g, editUrl).replace(/{alttext}/g, altText).replace(/{title}/g, title);
                                    }
                                    else {
                                        html = html.replace(/{url}/g, url).replace(/{alttext}/g, altText).replace(/{title}/g, title);
                                    }

                                    html = html.replace(/{externalsource}/g, externalSource);
                                    html = html.replace(/{mediastyle}/g, mediaClassText);
                                    html = html.replace(/{medianame}/g, mediaName);
                                    html = html.replace(/{mediatype}/g, mediaType);
                                    html = html.replace(/{siteurl}/g, siteUrl);

                                    html = html.replace(/{metadataclass}/g, "qbankmetadata_" + mediaType + "_" + templateId + "_" + templateName + "_" + mediaClassText);
                                    var range = activeEditor.dom.createRng();
                                    var wrapperTag = jQueryQ(qBankWrapperTag);
                                    if (wrapperTag.length > 0 && qBankWrapperTag != undefined && qBankWrapperTag != null) {
                                        var selBefore = activeEditor.selection.getContent();
                                        range.selectNode(wrapperTag[0]);
                                        activeEditor.selection.setRng(range);
                                    }

                                    console.info("## QBank - OnUpdate in 'dialogWindow.QbankPropertyLabelDisplay' - next we set the html content");

                                    if (activeEditor.getContent() == '') {
                                        // This is done to avoid IE bug
                                        activeEditor.setContent(html);
                                    }
                                    else {
                                        activeEditor.selection.setContent(html);
                                    }

                                    // To be able to move the media around in Tiny we need to resfresh th content
                                    activeEditor.setContent(activeEditor.getContent());

                                    console.info("## QBank - OnUpdate in 'dialogWindow.QbankPropertyLabelDisplay' - the html content is set,now we close");


                                    activeEditor.windowManager.close(window);
                                },
                                function () {
                                    window.parent.tinymce.activeEditor.windowManager.close(window);
                                }

                    );
            };

            // Select the Picker View
            self.showPicker = function () {
                var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                var dialogWindow = iframeWindow[0].contentWindow;
                dialogWindow.QbankShowPicker();
            },

            // Select the Property View
            self.showProperties = function (propertyValueList, mediaType) {
                var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                var dialogWindow = iframeWindow[0].contentWindow;

                dialogWindow.QbankPropertyValueUpdate(propertyValueList, mediaType);
            },

            // Media is selected and should now be converted. After that the Property View should be opened.
            self.convertAndShowProperties = function (qbankMedia, pickerTemplates) {

                console.info("## QBank: convertAndShowProperties");

                var qbankMediaId = qbankMedia.mediaId;
                var mediaType = "document";
                if (qbankMedia.mimetype.imagetemplate) {
                    mediaType = "image";
                }
                else if (qbankMedia.mimetype.videotemplate || qbankMedia.mimetype.classification == 'video') {
                    mediaType = "video";
                }

                var contentId = parseInt(epiPageId);

                jQueryQ("#loaderArea").show();

                var pickerTemplatesJson = JSON.stringify(pickerTemplates);
                var parameters = { 'currentContentId': contentId, 'episerverMediaId': 0, 'qbankMediaId': qbankMediaId, 'widgetId': 0, 'pickerTemplatesJson': pickerTemplatesJson, 'useTinyTemplate': true };
                jQueryQ.ajax({
                    url: appRootPath + "qbankmodule/Stores/qbankconverter/",
                    type: "GET",
                    timeout: 60000,
                    data: parameters,
                    contentType: "application/json",
                    dataType: "json",
                    dataFilter: jsonEPiServerStoreFilter,
                    success: function (convertedMedia) {

                        console.info("## QBank: convertAndShowProperties - Media converted");

                        jQueryQ("#loaderArea").hide();

                        var id = convertedMedia.id;
                        if (id != undefined && id != null) {
                            var iframeWindow = jQueryQ(".epi-legacyDialog").find("iframe");
                            if (iframeWindow.length > 0) {
                                var dialogWindow = iframeWindow[0].contentWindow;

                                console.info("## QBank: convertAndShowProperties - " + (dialogWindow.QbankPropertyValueUpdate ? "dialogWindow.QbankPropertyValueUpdate exist" : "dialogWindow.QbankPropertyValueUpdate does not exist"));

                                dialogWindow.QbankPropertyValueUpdate(convertedMedia.propertyValueList, mediaType);
                            }
                            else {
                                console.error("## QBank: convertAndShowProperties - no iframe found in epi-legacyDialog");
                            }
                        }
                        else {

                            console.error("## QBank: convertAndShowProperties - not convertedMedia.id");

                            alert(convertedMedia); // Error message
                        }
                    },
                    error: function (error) {

                        console.error("## QBank: convertAndShowProperties - Media was not converted");

                        alert("Media was not created!");
                        editor.windowManager.close(window);
                    }
                });
            },

            // Support: Get Property Value
            self.getPropertyValue = function (propertyValueList, key) {
                var value = jQueryQ.grep(propertyValueList, function (e) { return e.key == key; });
                if (value && value.length == 1) {
                    return value[0].value;
                }
                return "";
            },

            // Support: Get Page Id
            self.getPageId = function (context) {
                if (context == null) return "";

                // expression captures page id as group 1
                // expression captures page version as group 2
                var expression = new RegExp(/([0-9]{1,})_([0-9]{1,})/i);
                var pageContextArray = expression.exec(context.id);
                var pageId = "0";
                if (pageContextArray) {
                    pageId = pageContextArray[1]; // It´s a page
                }
                else if (context.parentId) {
                    pageId = context.parentId; // It´s a block
                }
                return pageId;
            };

            // Support: Get Media Id
            self.getMediaId = function (mediaUrl) {
                if (mediaUrl == null) return "";

                // expression captures page id as group 1
                var expression = new RegExp(/,,([0-9]{1,})/i);
                var matches = expression.exec(mediaUrl)
                return matches != null && matches.length > 0 ? matches[1] : '';
            };
        }

        // Public properties and methods
        return {
            init: function (ed) {
                supportObject = new support();
                editor = ed;

                var context = ed.settings.epi_page_context;
                if (context) {
                    epiPageId = supportObject.getPageId(context);
                }

                var baseAppRootPath = ed.baseURI.path;
                appRootPath = baseAppRootPath.substr(0, baseAppRootPath.indexOf("/CMS/")) + "/";
            },

            openDialog: function (wrapperTag, mediaUrl, callLimit) {
                qBankWrapperTag = wrapperTag;
                currentMediaUrl = mediaUrl;
                supportObject.waitForDialog(callLimit)
            },
        };

    }());

}(this, this.document, window.QBank = window.QBank || {}, epiJQuery)); // epiJQuery gives version 1.7.2, jQuery might give a later version



(function () {
    tinymce.create('tinymce.plugins.QBankModulePlugin', {
        /**
        * Initializes the plugin, this will be executed after the plugin    has been created.
        * This call is done before the editor instance has finished it's    initialization so use the onInit event
        * of the editor instance to intercept that event.
        *
        * @param {tinymce.Editor} ed Editor instance that the plugin is 
        initialized in.
        * @param {string} url Absolute URL to where the plugin is located
        */
        init: function (ed, url) {
            var t = this;
            t.editor = ed;
            QBank.tiny.init(ed);

            // Register the command so that it can be invoked
            ed.addCommand('qbankmodulecmd', function (ui, v) {
                tinymce.plugins.QBankModulePlugin.prototype._doCommand(ed);
            });

            // Register button
            ed.addButton('qbankmodulebtn', {
                title: 'QBank Media',
                cmd: 'qbankmodulecmd',
                image: url + '/img/qbank.png'
            });

            ed.onKeyUp.add(function (ed, e) {
                if (e.keyCode == 46 || e.keyCode == 8) { // Force delete
                    //if (ed.QBankWrapperTag != null && ed.QBankWrapperTag.length > 0) {
                    //    ed.QBankWrapperTag.remove();
                    //}
                    var sel = $(ed.selection.getNode()).closest('.qbankmediawrapper');
                    if (sel.length > 0) {
                        sel.remove();
                    }
                }
            });

            // Add a node change handler, selects the button in the UI when a // image is selected
            ed.onNodeChange.add(function (ed, cm, n, co) {
                //Since we can have other inline elements nested within the       
                //link we want to try get the closest link parent.
                //var img = ed.dom.getParent(n, 'img', ed.getBody());

                var node = ed.selection.getNode();
                var mediaTag = $(node).closest(".qbankmedia");
                var wrapperTag = $(node).closest(".qbankmediawrapper");

                if (wrapperTag.length == 0) {
                    wrapperTag = mediaTag
                }

                var isQBankMedia = (wrapperTag.length > 0);

                if (isQBankMedia && node.nodeName != "IMG" && node.nodeName != "OBJECT") {
                    var range = ed.dom.createRng();
                    range.selectNodeContents(wrapperTag[0]);
                    ed.selection.setRng(range)
                }

                var disableButton = !isQBankMedia && (node.nodeName == "IMG" || node.nodeName == "OBJECT") && !co;
                cm.setDisabled('qbankmodulebtn', disableButton);
                cm.setActive('qbankmodulebtn', isQBankMedia);
            });
        },

        _doCommand: function (ed) {
            var mediaSrc = '';

            var node = ed.selection.getNode();
            var mediaTag = $(node).find(".qbankmedia").andSelf();
            var wrapperTag = $(node).closest(".qbankmediawrapper");

            if (wrapperTag.length == 0) {
                wrapperTag = mediaTag
            }

            if (mediaTag.length == 0) {
                mediaTag = wrapperTag.find(".qbankmedia").andSelf();
            }

            if (mediaTag.length > 0) {
                mediaSrc = mediaTag.attr('src');
            }

            ed.windowManager.open({
                url: "../qbankmodule/ClientResources/QBankConnect.html",
                width: 900,
                height: 480
            });

            QBank.tiny.openDialog(wrapperTag[0], mediaSrc, 50);
        },

        /**
        * Returns information about the plugin as a name/value array.
        * The current keys are longname, author, authorurl, infourl and 
        version.
        * @return {Object} Name/value array containing information 
        about the plugin.
        */
        getInfo: function () {
            return {
                longname: 'QBank for EPiServer CMS',
                author: 'QBNK AB',
                authorurl: 'http://www.qbank.se',
                infourl: 'http://www.qbank.se',
                version: "3.0"
            };
        }

    });
    // Register plugin
    tinymce.PluginManager.add('qbankmodule',
       tinymce.plugins.QBankModulePlugin);
})();
